# Open iT Inc. Philippines
#
# DESCRIPTION:
# Fix licpoll2 Max In Use > Max Available.
# Adds a CHECK-IN for last checked-out FEATURE if a new CHECK-OUT with different
# client port is found.
#
# Reference: RM#7831
#
# Author: Ivan Bilon (ibilon@openit.com)
#

BEGIN {
	FS = ":"
	OFS = ":"
}

/@licpoll.log.version/ {
	split($0, a, "=")
	VERSION = a[2]
	Feature = 4
	if (VERSION >= 2.0 && VERSION < 2.2) {
		PRODUCT = 3
		FEATURE = 4
		FEATURE_VERSION = 5
		USER = 9
		USED = 7
		QUEUED = 8
		LICENSE_SERVER = 12
		LICENSE_SERVER_PORT = 13
		PORT = 14
	}
	else if (VERSION >= 2.2 && VERSION < 2.3) {
		PRODUCT = 3
		FEATURE = 4
		FEATURE_VERSION = 5
		USER = 11
		USED = 9
		QUEUED = 10
		LICENSE_SERVER = 14
		LICENSE_SERVER_PORT = 15
		PORT = 16
	}
	else if (VERSION >= 2.3 && VERSION < 2.4) {
		PRODUCT = 3
		FEATURE = 4
		FEATURE_VERSION = 5
		USER = 11
		USED = 9
		QUEUED = 10
		LICENSE_SERVER = 14
		LICENSE_SERVER_PORT = 15
		PORT = 16
	}
	else if (VERSION >= 2.4 && VERSION < 2.5) {
		PRODUCT = 3
		FEATURE = 4
		FEATURE_VERSION = 5
		USER = 12
		USED = 10
		QUEUED = 11
		LICENSE_SERVER = 15
		LICENSE_SERVER_PORT = 16
		PORT = 17
	}
	else if (VERSION >= 2.5) {
		PRODUCT = 3
		FEATURE = 4
		FEATURE_VERSION = 5
		USER = 12
		USED = 10
		QUEUED = 11
		LICENSE_SERVER = 15
		LICENSE_SERVER_PORT = 16
		PORT = 17
	}
}

/^@/ || !$0 {
	print
	next
}

$0 {
	if(polldata()) {
		if(checkout()) {
			checkinLast()
			add()
		} else {
			remove()
		}
	}

	
	print
}

END {
	
}

function polldata() {
	if($1 == 1)
		return 1
	return 0
}

function checkout() {
	if($USED == 0 && $QUEUED == 0)
		return 0
	return 1
}

function checkinLast() {
	if($USER SUBSEP $PRODUCT SUBSEP $FEATURE SUBSEP $FEATURE_VERSION SUBSEP $LICENSE_SERVER SUBSEP $LICENSE_SERVER_PORT SUBSEP $PORT in SET)
		return
	for(i in SET) {
		split(i, a, SUBSEP)
		user = a[1]
		port = a[7]
		if(user == $USER) {
			#print "CHECK-OUT", user, port
			print OtoI(SET[i])
			delete SET[i]
			return
		}
	}
}

function add() {
	SET[$USER, $PRODUCT, $FEATURE, $FEATURE_VERSION, $LICENSE_SERVER, $LICENSE_SERVER_PORT, $PORT] = $0
}

function remove() {
	delete SET[$USER, $PRODUCT, $FEATURE, $FEATURE_VERSION, $LICENSE_SERVER, $LICENSE_SERVER_PORT, $PORT]
}

function OtoI(record) {
	len = split(record, a, FS)
	a[USED] 	= 0 	# USED
	a[QUEUED] 	= 0 	# QUEUED
	str = ""
	for(j=1;j<=len;j++) {
		str = str OFS a[j]
	}
	return substr(str, 2)
}
