# Open iT Inc. Philippines
#
# This is an AWK script.
#
# Description:
# 	Fixes licpoll2 archive Max In Use > Max Available
# 	due to polled check-out package without check-in.
# 	Adds check-in record for packages, based on check-in
# 	records of a component (by a user), belonging to the package.
#
# Reference: RM#7831
#
# Author: Ivan Bilon (ibilon@openit.com)
#

BEGIN {
	FS = ":"
	OFS = ":"
	POLLS = 0
	POLL = 0
}

/@licpoll.log.version/ {
	split($0, a, "=")
	VERSION = a[2]
	Feature = 4
	if (VERSION >= 2.0 && VERSION < 2.2) {
		User 	= 9
		Used 	= 7
		Queued 	= 8
	}
	else if (VERSION >= 2.2 && VERSION < 2.3) {
		User 	= 11
		Used 	= 9
		Queued 	= 10
	}
	else if (VERSION >= 2.3 && VERSION < 2.4) {
		User 	= 11
		Used 	= 9
		Queued 	= 10
	}
	else if (VERSION >= 2.4 && VERSION < 2.5) {
		User 	= 12
		Used 	= 10
		Queued 	= 11
	}
	else if (VERSION >= 2.5) {
		User 	= 12
		Used 	= 10
		Queued 	= 11
	}
}

/@poller.daemon/ {
	POLLS++
	POLL = 1
}

POLL && ! $0 {
	POLL = 0
}

POLL {
	if($2 == 1)
		PACKAGE = 1
	else
		PACKAGE = 0
	if($2 == 2)
		COMPONENT = 1
	else
		COMPONENT = 0
	if($Used > 0 || $Queued > 0)
		CHECKOUT = 1
	else
		CHECKOUT = 0
	if($Used == 0 && $Queued == 0)
		CHECKIN = 1
	else
		CHECKIN = 0
}

POLL && PACKAGE && CHECKOUT {
	LPO[$Feature "^" $User] = $0 # LAST PACKAGE CHECK-OUT [FEATURE, USER]
}

POLL && PACKAGE && CHECKIN {
	delete LPO[$Feature "^" $User]
}

POLL && POLLS > 1 && COMPONENT && CHECKIN {
	LPOindex = has($Feature, $User)
	if(LPOindex) {
		print OtoI(LPO[LPOindex])
		delete LPO[LPOindex]
	}
}

{
	print
}


# -------- FUNCTIONS ---------



function has(feature, user) {
	for(i in LPO) {
		split(i, key, "^")
		F = key[1]
		U = key[2]
		if(index(feature, F) == 1 && user == U) {
			return i
		}
	}
	return ""
}

function OtoI(record) {
	len = split(record, a, FS)
	a[Used] 	= 0 	# USED
	a[Queued] 	= 0 	# QUEUED
	str = ""
	for(i=1;i<=len;i++) {
		str = str OFS a[i]
	}
	return substr(str, 2)
}
